package post

import (
	"context"
	"fmt"
	entity2 "github.com/milvus-io/milvus-sdk-go/v2/entity"
	"leafdev.top/Ecosystem/recommender/internal/entity"
)

func (s *Service) GetTag(c context.Context, name string) (*entity.Tag, error) {
	var tmq = s.dao.WithContext(c).TagMapping.Where(s.dao.TagMapping.Name.Eq(name))
	//Where(s.dao.TagMapping.ApplicationId.Eq(applicationEntity.Id.Uint()))

	tmqCount, err := tmq.Count()
	if err != nil {
		return nil, err
	}

	if tmqCount > 0 {
		r, err := tmq.First()

		if err != nil {
			return nil, err
		}

		return r.Tag, nil
	}

	t, err := s.dao.WithContext(c).Tag.Where(s.dao.Tag.Name.Eq(name)).
		//Where(s.dao.Tag.ApplicationId.Eq(applicationEntity.Id.Uint())).
		FirstOrCreate()

	if err != nil {
		return nil, err
	}

	if !t.Vectorized {
		err = s.SaveTagEmbedding(c, t)
	}

	return t, err
}

func (s *Service) HasBindTag(c context.Context, post *entity.Post, tagName string) (bool, error) {
	tag, err := s.GetTag(c, tagName)
	if err != nil {
		return false, err
	}

	count, err := s.dao.WithContext(c).PostTag.Where(s.dao.PostTag.PostId.Eq(post.Id.Uint())).
		Where(s.dao.PostTag.TagId.Eq(tag.Id.Uint())).Count()
	if err != nil {
		return false, err
	}

	return count > 0, nil
}

func (s *Service) BindTag(c context.Context, post *entity.Post, tagName string) error {
	tag, err := s.GetTag(c, tagName)
	if err != nil {
		return err
	}

	bind, err := s.HasBindTag(c, post, tag.Name)
	if err != nil {
		return err
	}

	if !bind {
		err = s.dao.WithContext(c).PostTag.Create(&entity.PostTag{
			PostId: &post.Id,
			TagId:  &tag.Id,
		})
	}

	return nil
}

func (s *Service) MarkAsProcessed(c context.Context, post *entity.Post) error {
	_, err := s.dao.WithContext(c).Post.Where(s.dao.Post.Id.Eq(post.Id.Uint())).Update(s.dao.Post.Processed, true)

	return err
}

func (s *Service) SaveTagEmbedding(c context.Context, tag *entity.Tag) error {
	emb, err := s.embedding.TextEmbedding(c, []string{tag.Name})
	if err != nil {
		return err
	}

	var entityCols = []entity2.Column{
		entity2.NewColumnFloatVector("vector", s.config.OpenAI.EmbeddingDim, emb),
		entity2.NewColumnInt64("tag_id", []int64{int64(tag.Id)}),
	}

	_, err = s.milvus.Upsert(c, s.config.Milvus.TagCollection, "", entityCols...)

	if err != nil {
		return err
	}

	_, err = s.dao.WithContext(c).Tag.Where(s.dao.Tag.Id.Eq(tag.Id.Uint())).Update(s.dao.Tag.Vectorized, true)

	return err
}

func (s *Service) SavePostEmbedding(c context.Context, post *entity.Post) error {
	tags, err := s.GetPostTags(c, post)
	if err != nil {
		return err
	}

	var tagString = ""
	for _, tag := range tags {
		tagString += tag.Name + " "
	}

	// 裁剪 > s.config.OpenAI.EmbeddingDim
	if len(tagString) > s.config.OpenAI.EmbeddingDim {
		tagString = tagString[:s.config.OpenAI.EmbeddingDim]
	}

	emb, err := s.embedding.TextEmbedding(c, []string{tagString})
	if err != nil {
		return err
	}

	var entityCols = []entity2.Column{
		entity2.NewColumnFloatVector("vector", s.config.OpenAI.EmbeddingDim, emb),
		entity2.NewColumnInt64("post_id", []int64{int64(post.Id)}),
		entity2.NewColumnInt64("category_id", []int64{int64(post.CategoryId.Uint())}),
		entity2.NewColumnInt64("application_id", []int64{int64(post.ApplicationId.Uint())}),
	}

	_, err = s.milvus.Upsert(c, s.config.Milvus.PostCollection, "", entityCols...)

	if err != nil {
		return err
	}

	_, err = s.dao.WithContext(c).Post.Where(s.dao.Post.Id.Eq(post.Id.Uint())).Update(s.dao.Post.Vectorized, true)

	return err
}

func (s *Service) RemovePostEmbedding(c context.Context, post *entity.Post) error {
	var filter = fmt.Sprintf(`psot_id == %d`, post.Id)
	errDelete := s.milvus.Delete(c, s.config.Milvus.PostCollection, "", filter)

	return errDelete
}
