package service

import (
	"leafdev.top/Ecosystem/recommender/internal/base/logger"
	"leafdev.top/Ecosystem/recommender/internal/service/application"
	"leafdev.top/Ecosystem/recommender/internal/service/auth"
	"leafdev.top/Ecosystem/recommender/internal/service/category"
	"leafdev.top/Ecosystem/recommender/internal/service/embedding"
	"leafdev.top/Ecosystem/recommender/internal/service/jwks"
	"leafdev.top/Ecosystem/recommender/internal/service/post"
	"leafdev.top/Ecosystem/recommender/internal/service/stream"
	"leafdev.top/Ecosystem/recommender/internal/service/user"

	"github.com/google/wire"
)

type Service struct {
	logger      *logger.Logger
	Jwks        *jwks.JWKS
	Stream      *stream.Service
	Auth        *auth.Service
	Application *application.Service
	Post        *post.Service
	Category    *category.Service
	User        *user.Service
	Embedding   *embedding.Service
}

var Provider = wire.NewSet(
	jwks.NewJWKS,
	embedding.NewService,
	stream.NewService,
	auth.NewAuthService,
	application.NewService,
	post.NewService,
	category.NewService,
	user.NewService,
	NewService,
)

func NewService(
	logger *logger.Logger,
	jwks *jwks.JWKS,
	stream *stream.Service,
	auth *auth.Service,
	application *application.Service,
	post *post.Service,
	category *category.Service,
	user *user.Service,
	embedding *embedding.Service,
) *Service {
	return &Service{
		logger,
		jwks,
		stream,
		auth,
		application,
		post,
		category,
		user,
		embedding,
	}
}
